/*
 * libmedia-service
 *
 * Copyright (c) 2000 - 2011 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Contact: Hyunjun Ko <zzoon.ko@samsung.com>, Haejeong Kim <backto.kim@samsung.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

#include <glib/gstdio.h>
#include <media-util-err.h>
#include "media-svc-media-folder.h"
#include "media-svc-debug.h"
#include "media-svc-env.h"
#include "media-svc-util.h"
#include "media-svc-db-utils.h"

int _media_svc_get_folder_id_by_foldername(sqlite3 *handle, const char *storage_id, const char *folder_name, char *folder_id, uid_t uid)
{
	int ret = MS_MEDIA_ERR_NONE;
	sqlite3_stmt *sql_stmt = NULL;
	char *sql = NULL;

	sql = sqlite3_mprintf("SELECT folder_id FROM %q WHERE storage_uuid = '%q' AND folder_path = '%q';", MEDIA_SVC_DB_TABLE_FOLDER, storage_id, folder_name);

	ret = _media_svc_sql_prepare_to_step(handle, sql, &sql_stmt);
	if (ret != MS_MEDIA_ERR_NONE) {
		if (ret == MS_MEDIA_ERR_DB_NO_RECORD)
			media_svc_debug("No record");
		else
			media_svc_error("error when _media_svc_get_folder_id_by_foldername. err = [%d]", ret);

		return ret;
	}

	SAFE_STRLCPY(folder_id, (const char *)sqlite3_column_text(sql_stmt, 0), MEDIA_SVC_UUID_SIZE + 1);

	SQLITE3_FINALIZE(sql_stmt);

	return ret;
}

static int __media_svc_append_folder(bool is_direct, const char *storage_id, ms_user_storage_type_e storage_type, const char *folder_id, const char *folder_path, uid_t uid)
{
	int ret = MS_MEDIA_ERR_NONE;
	char *folder_name = NULL;
	int folder_modified_date = 0;

	folder_name = g_path_get_basename(folder_path);
	folder_modified_date = _media_svc_get_file_time(folder_path);

	/*Update Pinyin If Support Pinyin*/
	char *folder_name_pinyin = NULL;
	if (_media_svc_check_pinyin_support())
		_media_svc_get_pinyin_str(folder_name, &folder_name_pinyin);
	/* Sometime SQLITE3 returns NO_RECORD, so need to consider conflict case.. */
	char *sql = sqlite3_mprintf("INSERT OR IGNORE INTO %q (folder_id, folder_path, folder_name, storage_uuid, folder_storage_type, folder_modified_time, folder_name_pinyin) values (%Q, %Q, %Q, %Q, '%d', '%d', %Q); ",
				MEDIA_SVC_DB_TABLE_FOLDER, folder_id, folder_path, folder_name, storage_id, storage_type, folder_modified_date, folder_name_pinyin);

	if (is_direct)
		ret = _media_svc_sql_query_direct(sql, uid);
	else
		ret = _media_svc_sql_query(sql, uid);

	SQLITE3_SAFE_FREE(sql);

	SAFE_FREE(folder_name);
	SAFE_FREE(folder_name_pinyin);

	return ret;
}

int _media_svc_update_folder_modified_time_by_folder_uuid(const char *folder_uuid, const char *folder_path, uid_t uid)
{
	int ret = MS_MEDIA_ERR_NONE;
	int modified_time = 0;

	modified_time = _media_svc_get_file_time(folder_path);

	char *sql = sqlite3_mprintf("UPDATE %q SET folder_modified_time=%d WHERE folder_id=%Q;", MEDIA_SVC_DB_TABLE_FOLDER, modified_time, folder_uuid);

	ret = _media_svc_sql_query(sql, uid);
	SQLITE3_SAFE_FREE(sql);

	return ret;
}

static int __media_svc_get_and_append_parent_folder(sqlite3 *handle, bool is_direct, const char *storage_id, const char *path, ms_user_storage_type_e storage_type, char *folder_id, uid_t uid)
{
	int ret = MS_MEDIA_ERR_NONE;
	unsigned int next_pos = 0;
	char *next = NULL;
	char *dir_path = NULL;
	const char *token = "/";
	char *folder_uuid = NULL;
	char tmp_folder_uuid[MEDIA_SVC_UUID_SIZE + 1] = {0, };
	bool folder_search_end = false;
	char *internal_path = NULL;

	memset(tmp_folder_uuid, 0, sizeof(tmp_folder_uuid));
	ret = ms_user_get_internal_root_path(uid, &internal_path);
	media_svc_retvm_if(ret != MS_MEDIA_ERR_NONE, ret, "Fail to get root path");

	if (STRING_VALID(internal_path) && (strncmp(path, internal_path, strlen(internal_path)) == 0))
		next_pos = strlen(internal_path);
	else if (STRING_VALID(MEDIA_ROOT_PATH_SDCARD) && strncmp(path, MEDIA_ROOT_PATH_SDCARD, strlen(MEDIA_ROOT_PATH_SDCARD)) == 0)
		next_pos = strlen(MEDIA_ROOT_PATH_SDCARD);
	else if (STRING_VALID(MEDIA_ROOT_PATH_EXTERNAL) && strncmp(path, MEDIA_ROOT_PATH_EXTERNAL, strlen(MEDIA_ROOT_PATH_EXTERNAL)) == 0)
		next_pos = strlen(MEDIA_ROOT_PATH_EXTERNAL);
	else if (STRING_VALID(MEDIA_ROOT_PATH_DISC) && strncmp(path, MEDIA_ROOT_PATH_DISC, strlen(MEDIA_ROOT_PATH_DISC)) == 0)
		next_pos = strlen(MEDIA_ROOT_PATH_DISC);
	else {
		media_svc_error("Invalid Path");
		media_svc_sec_error("Invalid Path [%s], internal_path [%s]", path, internal_path);
		SAFE_FREE(internal_path);
		return MS_MEDIA_ERR_INTERNAL;
	}

	SAFE_FREE(internal_path);

	while (!folder_search_end) {
		next = strstr(path + next_pos, token);
		if (next != NULL) {
			next_pos = (next - path);
			dir_path = strndup(path, next_pos);
			next_pos++;
		} else {
			dir_path = strndup(path, strlen(path));
			folder_search_end = true;
			media_svc_sec_debug("End Path[%s]", dir_path);
		}

		if (STRING_VALID(MEDIA_ROOT_PATH_EXTERNAL) && (g_strcmp0(dir_path, MEDIA_ROOT_PATH_EXTERNAL) == 0)) {
			/*To avoid insert MEDIA_ROOT_PATH_EXTERNAL path*/
			SAFE_FREE(dir_path);
			continue;
		}

		ret = _media_svc_get_folder_id_by_foldername(handle, storage_id, dir_path, tmp_folder_uuid, uid);
		if (ret == MS_MEDIA_ERR_DB_NO_RECORD) {
			folder_uuid = _media_info_generate_uuid();
			if (folder_uuid == NULL) {
				media_svc_error("Invalid UUID");
				SAFE_FREE(dir_path);
				return MS_MEDIA_ERR_INTERNAL;
			}

			ret = __media_svc_append_folder(is_direct, storage_id, storage_type, folder_uuid, dir_path, uid);
			if (ret != MS_MEDIA_ERR_NONE)
				media_svc_error("__media_svc_append_folder is failed");

			media_svc_sec_debug("Append new folder path[%s] uuid[%s]", dir_path, folder_uuid);
		}

		SAFE_FREE(dir_path);
	}

	if (STRING_VALID(folder_uuid)) {
		SAFE_STRLCPY(folder_id, folder_uuid, MEDIA_SVC_UUID_SIZE + 1);
	} else {
		media_svc_error("Fail to get folder_uuid");
		return MS_MEDIA_ERR_INTERNAL;
	}

	return MS_MEDIA_ERR_NONE;
}

int _media_svc_get_and_append_folder(sqlite3 *handle, bool is_direct, const char *storage_id, const char *path, ms_user_storage_type_e storage_type, char *folder_id, uid_t uid)
{
	int ret = MS_MEDIA_ERR_NONE;

	ret = _media_svc_get_folder_id_by_foldername(handle, storage_id, path, folder_id, uid);
	if (ret == MS_MEDIA_ERR_DB_NO_RECORD)
		ret = __media_svc_get_and_append_parent_folder(handle, is_direct, storage_id, path, storage_type, folder_id, uid);

	return ret;
}

int _media_svc_get_and_append_folder_id_by_path(sqlite3 *handle, bool is_direct, const char *storage_id, const char *path, ms_user_storage_type_e storage_type, char *folder_id, uid_t uid)
{
	char *dir_path = NULL;
	int ret = MS_MEDIA_ERR_NONE;

	dir_path = g_path_get_dirname(path);

	ret = _media_svc_get_and_append_folder(handle, is_direct, storage_id, dir_path, storage_type, folder_id, uid);

	SAFE_FREE(dir_path);

	return ret;
}

int _media_svc_get_and_append_folder_id_by_folder_path(sqlite3 *handle, const char *storage_id, const char *path, ms_user_storage_type_e storage_type, uid_t uid)
{
	char *path_name = NULL;
	int ret = MS_MEDIA_ERR_NONE;
	char *sql = NULL;
	char folder_id[MEDIA_SVC_UUID_SIZE + 1] = {0, };

	path_name = strdup(path);
	if (path_name == NULL) {
		media_svc_error("out of memory");
		return MS_MEDIA_ERR_OUT_OF_MEMORY;
	}

	ret = _media_svc_get_folder_id_by_foldername(handle, storage_id, path_name, folder_id, uid);
	if (ret == MS_MEDIA_ERR_DB_NO_RECORD) {
		ret = __media_svc_get_and_append_parent_folder(handle, true, storage_id, path_name, storage_type, folder_id, uid);
	} else {
		sql = sqlite3_mprintf("UPDATE %q SET validity=1 WHERE storage_uuid = '%q' AND folder_path = '%q';", MEDIA_SVC_DB_TABLE_FOLDER, storage_id, path);
		ret = _media_svc_sql_query_direct(sql, uid);
		SQLITE3_SAFE_FREE(sql);
	}

	SAFE_FREE(path_name);

	return ret;
}

int _media_svc_set_folder_validity(sqlite3 *handle, bool is_direct, const char *storage_id, const char *start_path, int validity, bool is_recursive, uid_t uid)
{
	int ret = MS_MEDIA_ERR_NONE;
	char *sql = NULL;

	if (is_recursive) {
		sql = sqlite3_mprintf("UPDATE %q SET validity=%d WHERE storage_uuid='%q' AND (folder_path LIKE '%q/%%' OR folder_path='%q');",
						MEDIA_SVC_DB_TABLE_FOLDER, validity, storage_id, start_path, start_path);
	} else {
		sql = sqlite3_mprintf("UPDATE %q SET validity=%d WHERE storage_uuid='%q' AND folder_path='%q';",
						MEDIA_SVC_DB_TABLE_FOLDER, validity, storage_id, start_path);
	}

	if (is_direct)
		ret = _media_svc_sql_query_direct(sql, uid);
	else
		ret = _media_svc_sql_query(sql, uid);

	SQLITE3_SAFE_FREE(sql);

	return ret;
}

int _media_svc_count_folder_with_path(sqlite3 *handle, const char *storage_id, const char *path, int *count)
{
	int ret = MS_MEDIA_ERR_NONE;
	sqlite3_stmt *sql_stmt = NULL;
	char *sql = NULL;

	media_svc_retvm_if(!STRING_VALID(storage_id), MS_MEDIA_ERR_INVALID_PARAMETER, "storage_id is NULL");
	media_svc_retvm_if(!STRING_VALID(path), MS_MEDIA_ERR_INVALID_PARAMETER, "path is NULL");

	sql = sqlite3_mprintf("SELECT count(*) FROM %q WHERE (storage_uuid='%q' AND folder_path='%q')", MEDIA_SVC_DB_TABLE_FOLDER, storage_id, path);

	ret = _media_svc_sql_prepare_to_step(handle, sql, &sql_stmt);

	media_svc_retv_if(ret != MS_MEDIA_ERR_NONE, ret);

	*count = sqlite3_column_int(sql_stmt, 0);

	SQLITE3_FINALIZE(sql_stmt);

	return MS_MEDIA_ERR_NONE;
}
