/*
 * libmedia-service
 *
 * Copyright (c) 2000 - 2011 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Contact: Hyunjun Ko <zzoon.ko@samsung.com>, Haejeong Kim <backto.kim@samsung.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <media-util-err.h>
#include "media-svc-media.h"
#include "media-svc-media-folder.h"
#include "media-svc-debug.h"
#include "media-svc-util.h"
#include "media-svc-db-utils.h"
#include "media-svc-noti.h"

#define MEDIA_SVC_MAX_COMMIT_SIZE 200

static __thread GList *g_media_svc_item_validity_query_list = NULL;
static __thread GList *g_media_svc_insert_item_query_list = NULL;
static __thread GList *g_media_svc_update_item_query_list = NULL;
static __thread GList *g_media_svc_update_list = NULL;

int _media_svc_count_record_with_path(sqlite3 *handle, const char *storage_id, const char *path, int *count)
{
	int ret = MS_MEDIA_ERR_NONE;
	sqlite3_stmt *sql_stmt = NULL;
	char *sql = NULL;

	media_svc_retvm_if(!STRING_VALID(storage_id), MS_MEDIA_ERR_INVALID_PARAMETER, "storage_id is NULL");
	media_svc_retvm_if(!STRING_VALID(path), MS_MEDIA_ERR_INVALID_PARAMETER, "Path is NULL");

	sql = sqlite3_mprintf("SELECT count(*) FROM '%q' WHERE media_path='%q'", storage_id, path);
	ret = _media_svc_sql_prepare_to_step(handle, sql, &sql_stmt);
	media_svc_retv_if(ret != MS_MEDIA_ERR_NONE, ret);

	*count = sqlite3_column_int(sql_stmt, 0);

	SQLITE3_FINALIZE(sql_stmt);

	return MS_MEDIA_ERR_NONE;
}

int _media_svc_get_modified_time(sqlite3 *handle, const char *storage_id, const char *path, int *modified_time)
{
	int ret = MS_MEDIA_ERR_NONE;
	sqlite3_stmt *sql_stmt = NULL;
	char *sql = NULL;

	media_svc_retvm_if(!STRING_VALID(storage_id), MS_MEDIA_ERR_INVALID_PARAMETER, "storage_id is NULL");
	media_svc_retvm_if(!STRING_VALID(path), MS_MEDIA_ERR_INVALID_PARAMETER, "Path is NULL");

	sql = sqlite3_mprintf("SELECT media_modified_time FROM '%q' WHERE media_path='%q'", storage_id, path);
	ret = _media_svc_sql_prepare_to_step(handle, sql, &sql_stmt);
	media_svc_retv_if(ret != MS_MEDIA_ERR_NONE, ret);

	*modified_time = sqlite3_column_int(sql_stmt, 0);

	SQLITE3_FINALIZE(sql_stmt);

	return MS_MEDIA_ERR_NONE;
}

int _media_svc_insert_item_with_data(bool is_direct, const char *storage_id, media_svc_content_info_s *content_info, bool stack_query, uid_t uid)
{
	int ret = MS_MEDIA_ERR_NONE;

	const char *db_fields = "media_id, media_path, media_display_name, media_type, media_mime_type, media_size, media_added_time, media_modified_time, folder_id, media_thumbnail_path, \
		media_title, album_id, media_album, media_artist, media_album_artist, media_genre, media_composer, media_year, media_recorded_date, media_copyright, media_track_num, media_description, \
		media_favourite, media_bitrate, media_bitpersample, media_samplerate, media_channel, media_duration, \
		media_longitude, media_latitude, media_altitude, exposure_time, fnumber, iso, model, media_width, media_height, media_datetaken, media_orientation, media_rating, media_is_drm, media_storage_type, media_timeline, sync_status, \
		media_file_name_pinyin, media_title_pinyin, media_album_pinyin, media_artist_pinyin, media_album_artist_pinyin, media_genre_pinyin, media_composer_pinyin, media_copyright_pinyin, media_description_pinyin, storage_uuid, media_360";

	const char *db_fields_basic = "media_id, media_path, media_display_name, media_type, media_mime_type, media_size, media_added_time, media_modified_time, folder_id, media_thumbnail_path, \
		album_id, media_favourite, media_is_drm, media_storage_type, media_timeline, sync_status, media_file_name_pinyin, storage_uuid, media_360";

	/* This sql is due to sqlite3_mprintf's wrong operation when using floating point in the text format */
	/* This code will be removed when sqlite3_mprintf works clearly */
	char *test_sql = sqlite3_mprintf("%f, %f, %f", content_info->media_meta.longitude, content_info->media_meta.latitude, content_info->media_meta.altitude);
	SQLITE3_SAFE_FREE(test_sql);

	/*Update Pinyin If Support Pinyin*/
	if (_media_svc_check_pinyin_support()) {
		if (STRING_VALID(content_info->file_name))
			_media_svc_get_pinyin_str(content_info->file_name, &content_info->file_name_pinyin);
		if (STRING_VALID(content_info->media_meta.title))
			_media_svc_get_pinyin_str(content_info->media_meta.title, &content_info->media_meta.title_pinyin);
		if (STRING_VALID(content_info->media_meta.album))
			_media_svc_get_pinyin_str(content_info->media_meta.album, &content_info->media_meta.album_pinyin);
		if (STRING_VALID(content_info->media_meta.artist))
			_media_svc_get_pinyin_str(content_info->media_meta.artist, &content_info->media_meta.artist_pinyin);
		if (STRING_VALID(content_info->media_meta.album_artist))
			_media_svc_get_pinyin_str(content_info->media_meta.album_artist, &content_info->media_meta.album_artist_pinyin);
		if (STRING_VALID(content_info->media_meta.genre))
			_media_svc_get_pinyin_str(content_info->media_meta.genre, &content_info->media_meta.genre_pinyin);
		if (STRING_VALID(content_info->media_meta.composer))
			_media_svc_get_pinyin_str(content_info->media_meta.composer, &content_info->media_meta.composer_pinyin);
		if (STRING_VALID(content_info->media_meta.copyright))
			_media_svc_get_pinyin_str(content_info->media_meta.copyright, &content_info->media_meta.copyright_pinyin);
		if (STRING_VALID(content_info->media_meta.description))
			_media_svc_get_pinyin_str(content_info->media_meta.description, &content_info->media_meta.description_pinyin);
	}

	char *sql = sqlite3_mprintf("INSERT INTO '%q' (%q) VALUES (%Q, %Q, %Q, %d, %Q, %lld, %d, %d, %Q, \
		%Q, %Q, %d, %Q, %Q, %Q, %Q, %Q, %Q, %Q, %Q, %Q, %Q, %d, \
		%d, %d, %d, %d, %d, %.6f, %.6f, %.6f, %Q, %.6f, %d, %Q, %d, %d, %Q, %d, \
		%d, %d, %d, %d, %d, %Q, %Q, %Q, %Q, %Q, %Q, %Q, %Q, %Q, %Q, %d);",
				content_info->storage_uuid, db_fields,
				content_info->media_uuid,
				content_info->path,
				content_info->file_name,
				content_info->media_type,
				content_info->mime_type,
				content_info->size,
				content_info->added_time,
				content_info->modified_time,
				content_info->folder_uuid,			/* 1st line */
				content_info->thumbnail_path,
				content_info->media_meta.title,
				content_info->album_id,
				content_info->media_meta.album,
				content_info->media_meta.artist,
				content_info->media_meta.album_artist,
				content_info->media_meta.genre,
				content_info->media_meta.composer,
				content_info->media_meta.year,
				content_info->media_meta.recorded_date,
				content_info->media_meta.copyright,
				content_info->media_meta.track_num,
				content_info->media_meta.description,
				content_info->favourate,		/* 2nd line */
				content_info->media_meta.bitrate,
				content_info->media_meta.bitpersample,
				content_info->media_meta.samplerate,
				content_info->media_meta.channel,
				content_info->media_meta.duration,
				content_info->media_meta.longitude,
				content_info->media_meta.latitude,
				content_info->media_meta.altitude,
				content_info->media_meta.exposure_time,
				content_info->media_meta.fnumber,
				content_info->media_meta.iso,
				content_info->media_meta.model,
				content_info->media_meta.width,
				content_info->media_meta.height,
				content_info->media_meta.datetaken,
				content_info->media_meta.orientation,		/* 3rd line */
				content_info->media_meta.rating,
				content_info->is_drm,
				content_info->storage_type,
				content_info->timeline,
				content_info->sync_status,
				content_info->file_name_pinyin,
				content_info->media_meta.title_pinyin,
				content_info->media_meta.album_pinyin,
				content_info->media_meta.artist_pinyin,
				content_info->media_meta.album_artist_pinyin,
				content_info->media_meta.genre_pinyin,
				content_info->media_meta.composer_pinyin,
				content_info->media_meta.copyright_pinyin,
				content_info->media_meta.description_pinyin,
				content_info->storage_uuid,
				content_info->media_meta.is_360		/* 4th line */
			);

	/* Check query length */
	if (sql != NULL && strlen(sql) >= (MEDIA_SVC_QUERY_LEN_MAX - 1) && is_direct == false) {
		media_svc_debug("Query length is over %d byte.. So, remake query using only basic information..", MEDIA_SVC_QUERY_LEN_MAX - 1);
		SQLITE3_SAFE_FREE(sql);

		sql = sqlite3_mprintf("INSERT INTO '%q' (%q) VALUES (%Q, %Q, %Q, %d, %Q, %lld, %d, %d, %Q, %Q, %d, %d, %d, %d, %d, %d, %d, %d, %Q, %Q, %d);",
						content_info->storage_uuid, db_fields_basic,
						content_info->media_uuid,
						content_info->path,
						content_info->file_name,
						content_info->media_type,
						content_info->mime_type,
						content_info->size,
						content_info->added_time,
						content_info->modified_time,
						content_info->folder_uuid,
						content_info->thumbnail_path,
						content_info->album_id,
						content_info->favourate,
						content_info->is_drm,
						content_info->storage_type,
						content_info->timeline,
						content_info->sync_status,
						content_info->file_name_pinyin,
						content_info->storage_uuid,
						content_info->media_meta.is_360
						);
	}

	if (!stack_query) {
		if (is_direct)
			ret = _media_svc_sql_query_direct(sql, uid);
		else
			ret = _media_svc_sql_query(sql, uid);

		SQLITE3_SAFE_FREE(sql);
		media_svc_retvm_if(ret != MS_MEDIA_ERR_NONE, ret, "failed to insert item");
	} else {
		media_svc_sec_debug("Query[%s]", sql);
		_media_svc_sql_query_add(&g_media_svc_insert_item_query_list, &sql);
	}

	return MS_MEDIA_ERR_NONE;
}

int _media_svc_update_meta_with_data(media_svc_content_info_s *content_info)
{
	int ret = MS_MEDIA_ERR_NONE;

	/* This sql is due to sqlite3_mprintf's wrong operation when using floating point in the text format */
	/* This code will be removed when sqlite3_mprintf works clearly */
	char *test_sql = sqlite3_mprintf("%f, %f, %f", content_info->media_meta.longitude, content_info->media_meta.latitude, content_info->media_meta.altitude);
	SQLITE3_SAFE_FREE(test_sql);

	char *sql = sqlite3_mprintf("UPDATE '%q' SET media_title=%Q, media_album=%Q, media_artist=%Q, media_album_artist=%Q, media_genre=%Q, media_composer=%Q, media_copyright=%Q, media_description=%Q WHERE media_path=%Q;",
				content_info->storage_uuid,
				content_info->media_meta.title,
				content_info->media_meta.album,
				content_info->media_meta.artist,
				content_info->media_meta.album_artist,
				content_info->media_meta.genre,
				content_info->media_meta.composer,
				content_info->media_meta.copyright,
				content_info->media_meta.description,
				content_info->path
				);

	if (sql != NULL) {
		media_svc_debug("query[%s]", sql);
		_media_svc_sql_query_add(&g_media_svc_update_item_query_list, &sql);
	} else {
		media_svc_error("sqlite3_mprintf failed");
		ret = MS_MEDIA_ERR_OUT_OF_MEMORY;
	}

	return ret;
}

int _media_svc_update_item_with_data(bool is_direct, const char *storage_id, media_svc_content_info_s *content_info, uid_t uid)
{
	int ret = MS_MEDIA_ERR_NONE;

	/* This sql is due to sqlite3_mprintf's wrong operation when using floating point in the text format */
	/* This code will be removed when sqlite3_mprintf works clearly */
	char *test_sql = sqlite3_mprintf("%f, %f, %f", content_info->media_meta.longitude, content_info->media_meta.latitude, content_info->media_meta.altitude);
	SQLITE3_SAFE_FREE(test_sql);

	/*Update Pinyin If Support Pinyin*/
	if (_media_svc_check_pinyin_support()) {
		if (STRING_VALID(content_info->file_name))
			_media_svc_get_pinyin_str(content_info->file_name, &content_info->file_name_pinyin);
		if (STRING_VALID(content_info->media_meta.title))
			_media_svc_get_pinyin_str(content_info->media_meta.title, &content_info->media_meta.title_pinyin);
		if (STRING_VALID(content_info->media_meta.album))
			_media_svc_get_pinyin_str(content_info->media_meta.album, &content_info->media_meta.album_pinyin);
		if (STRING_VALID(content_info->media_meta.artist))
			_media_svc_get_pinyin_str(content_info->media_meta.artist, &content_info->media_meta.artist_pinyin);
		if (STRING_VALID(content_info->media_meta.album_artist))
			_media_svc_get_pinyin_str(content_info->media_meta.album_artist, &content_info->media_meta.album_artist_pinyin);
		if (STRING_VALID(content_info->media_meta.genre))
			_media_svc_get_pinyin_str(content_info->media_meta.genre, &content_info->media_meta.genre_pinyin);
		if (STRING_VALID(content_info->media_meta.composer))
			_media_svc_get_pinyin_str(content_info->media_meta.composer, &content_info->media_meta.composer_pinyin);
		if (STRING_VALID(content_info->media_meta.copyright))
			_media_svc_get_pinyin_str(content_info->media_meta.copyright, &content_info->media_meta.copyright_pinyin);
		if (STRING_VALID(content_info->media_meta.description))
			_media_svc_get_pinyin_str(content_info->media_meta.description, &content_info->media_meta.description_pinyin);
	}

	char *sql = sqlite3_mprintf("UPDATE '%q' SET media_size=%lld, media_modified_time=%d, media_thumbnail_path=%Q, media_title=%Q, album_id=%d, media_album=%Q, media_artist=%Q, media_album_artist=%Q, media_genre=%Q, \
		media_composer=%Q, media_year=%Q, media_recorded_date=%Q, media_copyright=%Q, media_track_num=%Q, media_description=%Q, media_bitrate=%d, media_bitpersample=%d, media_samplerate=%d, media_channel=%d, media_duration=%d, \
		media_longitude=%f, media_latitude=%f, media_altitude=%f, exposure_time=%Q, fnumber=%f, iso=%d, model=%Q, media_width=%d, media_height=%d, media_datetaken=%Q, media_orientation=%d WHERE media_path=%Q;",
				storage_id,
				content_info->size,
				content_info->modified_time,
				content_info->thumbnail_path,
				content_info->media_meta.title,
				content_info->album_id,
				content_info->media_meta.album,
				content_info->media_meta.artist,
				content_info->media_meta.album_artist,
				content_info->media_meta.genre,
				content_info->media_meta.composer,
				content_info->media_meta.year,
				content_info->media_meta.recorded_date,
				content_info->media_meta.copyright,
				content_info->media_meta.track_num,
				content_info->media_meta.description,
				content_info->media_meta.bitrate,
				content_info->media_meta.bitpersample,
				content_info->media_meta.samplerate,
				content_info->media_meta.channel,
				content_info->media_meta.duration,
				content_info->media_meta.longitude,
				content_info->media_meta.latitude,
				content_info->media_meta.altitude,
				content_info->media_meta.exposure_time,
				content_info->media_meta.fnumber,
				content_info->media_meta.iso,
				content_info->media_meta.model,
				content_info->media_meta.width,
				content_info->media_meta.height,
				content_info->media_meta.datetaken,
				content_info->media_meta.orientation,
				content_info->path
				);

	/* Scanner use only batch insert */
	if (is_direct) {
		media_svc_sec_debug("Query [%s]", sql);
		_media_svc_sql_query_add(&g_media_svc_insert_item_query_list, &sql);
	} else {
		ret = _media_svc_sql_query(sql, uid);
		SQLITE3_SAFE_FREE(sql);
	}

	return ret;
}
int _media_svc_get_thumbnail_path_by_path(sqlite3 *handle, const char *path, char *thumbnail_path)
{
	int ret = MS_MEDIA_ERR_NONE;
	sqlite3_stmt *sql_stmt = NULL;
	char *sql = NULL;

	media_svc_retvm_if(!STRING_VALID(path), MS_MEDIA_ERR_INVALID_PARAMETER, "path is NULL");

	sql = sqlite3_mprintf("SELECT media_thumbnail_path FROM '%q' WHERE media_path='%q'", MEDIA_SVC_DB_VIEW_MEDIA, path);

	ret = _media_svc_sql_prepare_to_step(handle, sql, &sql_stmt);

	if (ret != MS_MEDIA_ERR_NONE) {
		if (ret == MS_MEDIA_ERR_DB_NO_RECORD)
			media_svc_debug("there is no thumbnail.");
		else
			media_svc_error("error when _media_svc_get_thumbnail_path_by_path. err = [%d]", ret);

		return ret;
	}

	SAFE_STRLCPY(thumbnail_path, (const char *)sqlite3_column_text(sql_stmt, 0), MEDIA_SVC_PATHNAME_SIZE);

	SQLITE3_FINALIZE(sql_stmt);

	return MS_MEDIA_ERR_NONE;
}

int _media_svc_get_media_type_by_path(sqlite3 *handle, const char *storage_id, const char *path, int *media_type)
{
	int ret = MS_MEDIA_ERR_NONE;
	sqlite3_stmt *sql_stmt = NULL;
	char *sql = NULL;

	media_svc_retvm_if(!STRING_VALID(storage_id), MS_MEDIA_ERR_INVALID_PARAMETER, "storage_id is NULL");
	media_svc_retvm_if(!STRING_VALID(path), MS_MEDIA_ERR_INVALID_PARAMETER, "path is NULL");

	sql = sqlite3_mprintf("SELECT media_type FROM '%q' WHERE media_path='%q'", storage_id, path);

	ret = _media_svc_sql_prepare_to_step(handle, sql, &sql_stmt);

	if (ret != MS_MEDIA_ERR_NONE) {
		media_svc_error("error when _media_svc_get_media_type_by_path. err = [%d]", ret);
		return ret;
	}

	*media_type = sqlite3_column_int(sql_stmt, 0);

	SQLITE3_FINALIZE(sql_stmt);

	return MS_MEDIA_ERR_NONE;
}

int _media_svc_delete_item_by_path(const char *storage_id, const char *path, uid_t uid)
{
	int ret = MS_MEDIA_ERR_NONE;
	char *sql = NULL;

	media_svc_retvm_if(!STRING_VALID(storage_id), MS_MEDIA_ERR_INVALID_PARAMETER, "storage_id is NULL");
	media_svc_retvm_if(!STRING_VALID(path), MS_MEDIA_ERR_INVALID_PARAMETER, "path is NULL");

	sql = sqlite3_mprintf("DELETE FROM '%q' WHERE media_path='%q';", storage_id, path);

	ret = _media_svc_sql_query(sql, uid);
	SQLITE3_SAFE_FREE(sql);
	media_svc_retvm_if(ret != MS_MEDIA_ERR_NONE, ret, "failed to delete item");

	return ret;
}

int _media_svc_update_item_validity(const char *storage_id, const char *path, int validity, bool stack_query, uid_t uid)
{
	int ret = MS_MEDIA_ERR_NONE;
	char *sql = NULL;

	media_svc_retvm_if(!STRING_VALID(storage_id), MS_MEDIA_ERR_INVALID_PARAMETER, "storage_id is NULL");
	media_svc_retvm_if(!STRING_VALID(path), MS_MEDIA_ERR_INVALID_PARAMETER, "path is NULL");

	media_svc_debug("path=[%s], validity=[%d]", path, validity);

	sql = sqlite3_mprintf("UPDATE '%q' SET validity=%d WHERE media_path='%q';", storage_id, validity, path);

	if (!stack_query) {
		ret = _media_svc_sql_query_direct(sql, uid);
		SQLITE3_SAFE_FREE(sql);
	} else {
		_media_svc_sql_query_add(&g_media_svc_item_validity_query_list, &sql);
	}

	return ret;
}

int _media_svc_update_thumbnail_path(const char *storage_id, const char *path, const char *thumb_path, uid_t uid)
{
	int ret = MS_MEDIA_ERR_NONE;

	char *sql = sqlite3_mprintf("UPDATE '%q' SET media_thumbnail_path=%Q WHERE media_path=%Q;", storage_id, thumb_path, path);

	ret = _media_svc_sql_query(sql, uid);
	SQLITE3_SAFE_FREE(sql);

	return ret;
}

int _media_svc_update_item_by_path(const char *org_storage_id, const char *src_path, const char *dst_storage_id, ms_user_storage_type_e dest_storage, const char *dest_path, const char *file_name, int modified_time, const char *folder_uuid, uid_t uid)
{
	/* update path, filename, modified_time, folder_uuid, thumbnail_path, */

	/* NOTICE (This operation is abnormal. Modifications should be cautious.)
	1. Update source to destination info
	2. Insert destination table
	3. Remove source info
	*/
	int ret = MS_MEDIA_ERR_NONE;
	char *update = sqlite3_mprintf("UPDATE '%q' SET media_path=%Q, media_display_name=%Q, media_modified_time=%d, folder_id=%Q, media_storage_type=%d, storage_uuid='%q' WHERE media_path=%Q;",
				org_storage_id, dest_path, file_name, modified_time, folder_uuid, dest_storage, dst_storage_id, src_path);
	char *insert = sqlite3_mprintf("INSERT OR REPLACE INTO '%q' SELECT * FROM '%q' WHERE media_path = '%q';", dst_storage_id, org_storage_id, dest_path);
	char *remove = sqlite3_mprintf("DELETE FROM '%q' WHERE media_path = '%q';", org_storage_id, dest_path);
	char *query = NULL;

	if (strcmp(org_storage_id, dst_storage_id) != 0)
		query = sqlite3_mprintf("%s %s %s", update, insert, remove);
	else
		query = sqlite3_mprintf("%s %s", update, insert);

	SQLITE3_SAFE_FREE(update);
	SQLITE3_SAFE_FREE(insert);
	SQLITE3_SAFE_FREE(remove);

	ret = _media_svc_sql_query(query, uid);
	SQLITE3_SAFE_FREE(query);

	return ret;
}

int _media_svc_list_query_do(media_svc_query_type_e query_type, uid_t uid)
{
	int ret = MS_MEDIA_ERR_NONE;

	/* For multiples of 200, empty requests are possible */
	switch (query_type) {
	case MEDIA_SVC_QUERY_UPDATE_COMMON:
		if (g_media_svc_update_list == NULL || g_list_length(g_media_svc_update_list) == 0)
			return MS_MEDIA_ERR_NONE;

		ret = _media_svc_sql_query_list(&g_media_svc_update_list, uid);
		break;
	case MEDIA_SVC_QUERY_SET_ITEM_VALIDITY:
		ret = _media_svc_sql_query_list_direct(&g_media_svc_item_validity_query_list, uid);
		break;
	case MEDIA_SVC_QUERY_INSERT_ITEM:
		ret = _media_svc_sql_query_list_direct(&g_media_svc_insert_item_query_list, uid);
		break;
	case MEDIA_SVC_QUERY_UPDATE_ITEM:
		ret = _media_svc_sql_query_list_direct(&g_media_svc_update_item_query_list, uid);
		break;
	default:
		media_svc_error("Wrong type[%d]", query_type);
		return MS_MEDIA_ERR_INVALID_PARAMETER;
	}

	media_svc_retvm_if(ret != MS_MEDIA_ERR_NONE, ret, "Failed to request queries");

	return MS_MEDIA_ERR_NONE;
}

int _media_svc_append_query_list(const char *query, uid_t uid)
{
	int ret = MS_MEDIA_ERR_NONE;

	media_svc_retvm_if(query == NULL, MS_MEDIA_ERR_INVALID_PARAMETER, "query is NULL");

	g_media_svc_update_list = g_list_append(g_media_svc_update_list, (gpointer)query);

	if (g_list_length(g_media_svc_update_list) >= MEDIA_SVC_MAX_COMMIT_SIZE)
		ret = _media_svc_list_query_do(MEDIA_SVC_QUERY_UPDATE_COMMON, uid);

	return ret;
}

int _media_svc_get_media(sqlite3 *handle, const char *sql, GArray **path_list)
{
	int ret = MS_MEDIA_ERR_NONE;
	sqlite3_stmt *sql_stmt = NULL;
	media_svc_file_s *file_info = NULL;

	media_svc_retvm_if(!sql, MS_MEDIA_ERR_INVALID_PARAMETER, "query is NULL");
	media_svc_retvm_if(!path_list, MS_MEDIA_ERR_INVALID_PARAMETER, "array is NULL");

	ret = _media_svc_sql_prepare_to_step_simple(handle, sql, &sql_stmt);
	media_svc_retvm_if(ret != MS_MEDIA_ERR_NONE, ret, "_media_svc_sql_prepare_to_step_simple() failed [%d]", ret);
	while (sqlite3_step(sql_stmt) == SQLITE_ROW) {
		file_info = malloc(sizeof(media_svc_file_s));
		if (!file_info) {
			media_svc_error("Allcation failed");
			continue;
		}

		SAFE_STRLCPY(file_info->path, (char *)sqlite3_column_text(sql_stmt, 0), sizeof(file_info->path));
		SAFE_STRLCPY(file_info->storage_id, (char *)sqlite3_column_text(sql_stmt, 1), sizeof(file_info->storage_id));
		g_array_append_val(*path_list, file_info);
	}

	SQLITE3_FINALIZE(sql_stmt);

	return ret;
}

int _media_svc_get_noti_info(sqlite3 *handle, const char *storage_id, const char *path, media_svc_noti_item **item)
{
	int ret = MS_MEDIA_ERR_NONE;
	sqlite3_stmt *sql_stmt = NULL;
	char *sql = NULL;

	media_svc_retvm_if(item == NULL, MS_MEDIA_ERR_INVALID_PARAMETER, "item is NULL");
	media_svc_retvm_if(!STRING_VALID(storage_id), MS_MEDIA_ERR_INVALID_PARAMETER, "storage_id is NULL");
	media_svc_retvm_if(!STRING_VALID(path), MS_MEDIA_ERR_INVALID_PARAMETER, "path is NULL");

	sql = sqlite3_mprintf("SELECT media_id, media_type, media_mime_type FROM '%q' WHERE media_path=%Q", storage_id, path);
	ret = _media_svc_sql_prepare_to_step(handle, sql, &sql_stmt);
	media_svc_retvm_if(ret != MS_MEDIA_ERR_NONE, ret, "_media_svc_sql_prepare_to_step() failed [%d]", ret);

	*item = calloc(1, sizeof(media_svc_noti_item));
	if (*item == NULL) {
		media_svc_error("Allocation failed");
		return MS_MEDIA_ERR_OUT_OF_MEMORY;
	}

	(*item)->media_uuid = g_strdup((const char *)sqlite3_column_text(sql_stmt, 0));
	(*item)->media_type = sqlite3_column_int(sql_stmt, 1);
	(*item)->mime_type = g_strdup((const char *)sqlite3_column_text(sql_stmt, 2));

	SQLITE3_FINALIZE(sql_stmt);

	return MS_MEDIA_ERR_NONE;
}
