/*
 * libmedia-service
 *
 * Copyright (c) 2000 - 2011 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Contact: Hyunjun Ko <zzoon.ko@samsung.com>, Haejeong Kim <backto.kim@samsung.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

#include "media-util-err.h"
#include "media-svc-debug.h"
#include "media-svc-env.h"
#include "media-svc-db-utils.h"
#include "media-svc-util.h"
#include "media-svc-storage.h"

int _media_svc_check_storage(sqlite3 *handle, const char *storage_id, char **storage_path, int *validity, uid_t uid)
{
	int ret = MS_MEDIA_ERR_NONE;
	sqlite3_stmt *sql_stmt = NULL;
	char *sql = NULL;

	media_svc_retvm_if(storage_id == NULL, MS_MEDIA_ERR_INVALID_PARAMETER, "storage_id is NULL");
	media_svc_retvm_if(storage_path == NULL, MS_MEDIA_ERR_INVALID_PARAMETER, "storage_path is NULL");
	media_svc_retvm_if(validity == NULL, MS_MEDIA_ERR_INVALID_PARAMETER, "validity is NULL");

	*storage_path = NULL;
	*validity = 0;

	sql = sqlite3_mprintf("SELECT storage_path, validity FROM '%q' WHERE storage_id=%Q", MEDIA_SVC_DB_TABLE_STORAGE, storage_id);
	ret = _media_svc_sql_prepare_to_step(handle, sql, &sql_stmt);
	media_svc_retv_if(ret != MS_MEDIA_ERR_NONE, ret);

	*storage_path = g_strdup((const char *)sqlite3_column_text(sql_stmt, 0));
	*validity = sqlite3_column_int(sql_stmt, 1);

	SQLITE3_FINALIZE(sql_stmt);

	/*check storage media table*/
	if (STRING_VALID(storage_id)) {
		int table_cnt = 0;

		/*Select list of storage*/
		sql = sqlite3_mprintf("SELECT COUNT(*) FROM SQLITE_MASTER WHERE type='table' and name='%q'", storage_id);
		ret = _media_svc_sql_prepare_to_step(handle, sql, &sql_stmt);
		media_svc_retv_if(ret != MS_MEDIA_ERR_NONE, ret);

		table_cnt = sqlite3_column_int(sql_stmt, 0);
		SQLITE3_FINALIZE(sql_stmt);

		if (table_cnt > 0) {
			/*DO NOT THING*/
		} else {
			media_svc_error("media table not exist for storage [%s]", storage_id);
			/*make storage media table*/
			ret = _media_svc_create_media_table_with_id(storage_id, uid);
			media_svc_retvm_if(ret != MS_MEDIA_ERR_NONE, ret, "create media table failed : %d", ret);
		}
	}

	return MS_MEDIA_ERR_NONE;
}

int _media_svc_append_storage(const char *storage_id, const char *storage_path, ms_user_storage_type_e storage_type, uid_t uid)
{
	int ret = MS_MEDIA_ERR_NONE;
	char *sql = sqlite3_mprintf("INSERT INTO %q (storage_id, storage_path, storage_type) values (%Q, %Q, %d);",
						MEDIA_SVC_DB_TABLE_STORAGE, storage_id, storage_path, storage_type);

	ret = _media_svc_sql_query_direct(sql, uid);
	SQLITE3_SAFE_FREE(sql);

	return ret;
}

int _media_svc_update_storage_path(sqlite3 *handle, const char *storage_id, const char *path, uid_t uid)
{
	int ret = MS_MEDIA_ERR_NONE;
	char *sql = NULL;
	char *old_storage_path = NULL;
	int validity = 0;

	media_svc_retvm_if(storage_id == NULL, MS_MEDIA_ERR_INVALID_PARAMETER, "storage_id is NULL");
	media_svc_retvm_if(path == NULL, MS_MEDIA_ERR_INVALID_PARAMETER, "path is NULL");

	/*Get old path*/
	ret = _media_svc_check_storage(handle, storage_id, &old_storage_path, &validity, uid);
	media_svc_retv_if(ret != MS_MEDIA_ERR_NONE, ret);

	/*Storage table update*/
	sql = sqlite3_mprintf("UPDATE '%q' SET storage_path=%Q WHERE storage_id=%Q;", MEDIA_SVC_DB_TABLE_STORAGE, path, storage_id);
	ret = _media_svc_sql_query_direct(sql, uid);
	SQLITE3_SAFE_FREE(sql);
	if (ret != MS_MEDIA_ERR_NONE) {
		g_free(old_storage_path);
		return ret;
	}

	/*Folder table update*/
	sql = sqlite3_mprintf("UPDATE '%q' SET folder_path=REPLACE(folder_path, %Q, %Q) WHERE storage_uuid=%Q;", MEDIA_SVC_DB_TABLE_FOLDER, old_storage_path, path, storage_id);
	ret = _media_svc_sql_query_direct(sql, uid);
	SQLITE3_SAFE_FREE(sql);
	if (ret != MS_MEDIA_ERR_NONE) {
		g_free(old_storage_path);
		return ret;
	}

	/*Media table update*/
	sql = sqlite3_mprintf("UPDATE '%q' SET media_path=REPLACE(media_path, %Q, %Q);", storage_id, old_storage_path, path);
	ret = _media_svc_sql_query_direct(sql, uid);
	SQLITE3_SAFE_FREE(sql);
	g_free(old_storage_path);
	media_svc_retv_if(ret != MS_MEDIA_ERR_NONE, ret);

	return ret;
}

int __media_svc_delete_thumbnail(sqlite3 *handle, const char *storage_id)
{
	int ret = MS_MEDIA_ERR_NONE;
	char *sql = NULL;
	sqlite3_stmt *sql_stmt = NULL;

	sql = sqlite3_mprintf("SELECT media_thumbnail_path FROM '%q' WHERE media_thumbnail_path is not null;", storage_id);
	ret = _media_svc_sql_prepare_to_step_simple(handle, sql, &sql_stmt);
	media_svc_retv_if(ret != MS_MEDIA_ERR_NONE, ret);

	while (sqlite3_step(sql_stmt) == SQLITE_ROW)
		_media_svc_remove_file((const char *)sqlite3_column_text(sql_stmt, 0));

	SQLITE3_FINALIZE(sql_stmt);

	return ret;
}

int _media_svc_delete_invalid_storage(sqlite3 *handle, uid_t uid)
{
	int ret = MS_MEDIA_ERR_NONE;
	char *sql = NULL;
	char *storage_id = NULL;
	sqlite3_stmt *sql_stmt = NULL;
	GPtrArray *storage_list = NULL;
	int i = 0;

	sql = sqlite3_mprintf("SELECT storage_id FROM '%q' WHERE validity=0;", MEDIA_SVC_DB_TABLE_STORAGE);
	ret = _media_svc_sql_prepare_to_step_simple(handle, sql, &sql_stmt);
	media_svc_retv_if(ret != MS_MEDIA_ERR_NONE, ret);

	storage_list = g_ptr_array_new_with_free_func(g_free);

	while (sqlite3_step(sql_stmt) == SQLITE_ROW) {
		storage_id = g_strdup((const char *)sqlite3_column_text(sql_stmt, 0));
		if (storage_id != NULL)
			g_ptr_array_add(storage_list, storage_id);
	}

	SQLITE3_FINALIZE(sql_stmt);

	for (i = 0; i < storage_list->len; i++) {
		storage_id = g_ptr_array_index(storage_list, i);

		ret = __media_svc_delete_thumbnail(handle, storage_id);
		if (ret != MS_MEDIA_ERR_NONE)
			media_svc_error("Fail to remove thumbnail");

		/* remove media before drop table (for clear playlist, and tag table)*/
		sql = sqlite3_mprintf("DELETE FROM '%q';DROP TABLE '%q';", storage_id, storage_id);
		ret = _media_svc_sql_query_direct(sql, uid);
		SQLITE3_SAFE_FREE(sql);
		if (ret != MS_MEDIA_ERR_NONE)
			media_svc_error("Fail to drop table[%s]", storage_id);
	}

	/* Update storage, folder table */
	sql = sqlite3_mprintf("DELETE FROM %q WHERE validity=0;DELETE FROM %q WHERE validity=0;", MEDIA_SVC_DB_TABLE_STORAGE, MEDIA_SVC_DB_TABLE_FOLDER);
	ret = _media_svc_sql_query_direct(sql, uid);
	SQLITE3_SAFE_FREE(sql);
	if (ret != MS_MEDIA_ERR_NONE)
		media_svc_error("Fail to update storage table");

	g_ptr_array_free(storage_list, TRUE);

	return ret;
}

int _media_svc_update_storage_validity(const char *storage_id, int validity, uid_t uid)
{
	int ret = MS_MEDIA_ERR_NONE;
	char *sql = NULL;

	if (storage_id == NULL)
		sql = sqlite3_mprintf("UPDATE '%q' SET validity=%d;", MEDIA_SVC_DB_TABLE_STORAGE, validity);
	else
		sql = sqlite3_mprintf("UPDATE '%q' SET validity=%d WHERE storage_id=%Q;", MEDIA_SVC_DB_TABLE_STORAGE, validity, storage_id);

	ret = _media_svc_sql_query_direct(sql, uid);
	SQLITE3_SAFE_FREE(sql);

	return ret;
}

int _media_svc_get_storage_uuid(sqlite3 *handle, const char *path, char *storage_id, uid_t uid)
{
	int ret = MS_MEDIA_ERR_NONE;
	sqlite3_stmt *sql_stmt = NULL;
	char *sql = NULL;
	char *storage_path = NULL;
	char *remain_path = NULL;
	int remain_len = 0;
	char *internal_path = NULL;

	media_svc_retvm_if(path == NULL, MS_MEDIA_ERR_INVALID_PARAMETER, "path is NULL");

	ret = ms_user_get_internal_root_path(uid, &internal_path);
	media_svc_retvm_if(ret != MS_MEDIA_ERR_NONE, ret, "Fail to get root path");

	if (STRING_VALID(internal_path) && strncmp(path, internal_path, strlen(internal_path)) == 0) {
		SAFE_STRLCPY(storage_id, MEDIA_SVC_DB_TABLE_MEDIA, MEDIA_SVC_UUID_SIZE+1);
		SAFE_FREE(internal_path);
		return MS_MEDIA_ERR_NONE;
	}

	SAFE_FREE(internal_path);

	remain_path = strstr(path + (STRING_VALID(MEDIA_ROOT_PATH_USB) ? strlen(MEDIA_ROOT_PATH_USB) : 0) + 1, "/");
	if (remain_path != NULL)
		remain_len = strlen(remain_path);

	storage_path = strndup(path, strlen(path) - remain_len);

	sql = sqlite3_mprintf("SELECT storage_id FROM '%q' WHERE validity=1 AND storage_path = '%q'", MEDIA_SVC_DB_TABLE_STORAGE, storage_path);

	ret = _media_svc_sql_prepare_to_step(handle, sql, &sql_stmt);
	SAFE_FREE(storage_path);
	media_svc_retv_if(ret != MS_MEDIA_ERR_NONE, ret);

	if (STRING_VALID((const char *)sqlite3_column_text(sql_stmt, 0)))
		SAFE_STRLCPY(storage_id, (const char *)sqlite3_column_text(sql_stmt, 0), MEDIA_SVC_UUID_SIZE+1);

	SQLITE3_FINALIZE(sql_stmt);

	if (!STRING_VALID(storage_id)) {
		media_svc_error("Not found valid storage id [%s]", path);
		ret = MS_MEDIA_ERR_INVALID_PARAMETER;
	}

	return ret;
}

int _media_svc_get_storage_path(sqlite3 *handle, GPtrArray **storage_path)
{
	int ret = MS_MEDIA_ERR_NONE;
	sqlite3_stmt *sql_stmt = NULL;
	char *sql = NULL;
	char *root_path = NULL;

	media_svc_retvm_if(!storage_path, MS_MEDIA_ERR_INVALID_PARAMETER, "storage_path is NULL");

	sql = sqlite3_mprintf("SELECT storage_path FROM %q WHERE validity=1", MEDIA_SVC_DB_TABLE_STORAGE);

	ret = _media_svc_sql_prepare_to_step_simple(handle, sql, &sql_stmt);
	media_svc_retv_if(ret != MS_MEDIA_ERR_NONE, ret);

	while (sqlite3_step(sql_stmt) == SQLITE_ROW) {
		root_path = g_strdup((const char *)sqlite3_column_text(sql_stmt, 0));
		g_ptr_array_add(*storage_path, root_path);
	}

	SQLITE3_FINALIZE(sql_stmt);

	return ret;
}

