/*
 * libmedia-service
 *
 * Copyright (c) 2000 - 2011 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Contact: Hyunjun Ko <zzoon.ko@samsung.com>, Haejeong Kim <backto.kim@samsung.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

#include <string.h>
#include <errno.h>
#include "media-svc.h"
#include "media-svc-media.h"
#include "media-svc-debug.h"
#include "media-svc-util.h"
#include "media-svc-db-utils.h"
#include "media-svc-env.h"
#include "media-svc-media-folder.h"
#include "media-svc-album.h"
#include "media-svc-noti.h"
#include "media-svc-storage.h"

static __thread int g_media_svc_item_validity_data_cnt = 1;
static __thread int g_media_svc_item_validity_cur_data_cnt = 0;

static __thread int g_media_svc_insert_item_data_cnt = 1;
static __thread int g_media_svc_insert_item_cur_data_cnt = 0;

/* Flag for items to be published by notification */
static __thread bool g_insert_with_noti = false;

#define BATCH_REQUEST_MAX 300

int media_svc_get_user_version(sqlite3 *handle, int *user_version)
{
	return _media_svc_get_user_version(handle, user_version);
}

int media_svc_create_table(uid_t uid)
{
	int ret = MS_MEDIA_ERR_NONE;
	char *sql = NULL;

	media_svc_debug_fenter();

	ret = _media_svc_init_table_query(MEDIA_SVC_DB_TABLE_MEDIA);
	if (ret != MS_MEDIA_ERR_NONE) {
		media_svc_error("_media_svc_init_table_query fail.");
		goto ERROR;
	}

	/*create media table*/
	ret = _media_svc_make_table_query(MEDIA_SVC_DB_TABLE_MEDIA, MEDIA_SVC_DB_LIST_MEDIA, uid);
	if (ret != MS_MEDIA_ERR_NONE) {
		media_svc_error("_media_svc_make_table_query fail.");
		goto ERROR;
	}

	/*create folder table*/
	ret = _media_svc_make_table_query(MEDIA_SVC_DB_TABLE_FOLDER, MEDIA_SVC_DB_LIST_FOLDER, uid);
	if (ret != MS_MEDIA_ERR_NONE) {
		media_svc_error("_media_svc_make_table_query fail.");
		goto ERROR;
	}

	/*create playlist_map table*/
	ret = _media_svc_make_table_query(MEDIA_SVC_DB_TABLE_PLAYLIST_MAP, MEDIA_SVC_DB_LIST_PLAYLIST_MAP, uid);
	if (ret != MS_MEDIA_ERR_NONE) {
		media_svc_error("_media_svc_make_table_query fail.");
		goto ERROR;
	}

	/*create playlist table*/
	ret = _media_svc_make_table_query(MEDIA_SVC_DB_TABLE_PLAYLIST, MEDIA_SVC_DB_LIST_PLAYLIST, uid);
	if (ret != MS_MEDIA_ERR_NONE) {
		media_svc_error("_media_svc_make_table_query fail.");
		goto ERROR;
	}

	/* create album table*/
	ret = _media_svc_make_table_query(MEDIA_SVC_DB_TABLE_ALBUM, MEDIA_SVC_DB_LIST_ALBUM, uid);
	if (ret != MS_MEDIA_ERR_NONE) {
		media_svc_error("_media_svc_make_table_query fail.");
		goto ERROR;
	}

	/*create tag_map table*/
	ret = _media_svc_make_table_query(MEDIA_SVC_DB_TABLE_TAG_MAP, MEDIA_SVC_DB_LIST_TAG_MAP, uid);
	if (ret != MS_MEDIA_ERR_NONE) {
		media_svc_error("_media_svc_make_table_query fail.");
		goto ERROR;
	}

	/*create tag table*/
	ret = _media_svc_make_table_query(MEDIA_SVC_DB_TABLE_TAG, MEDIA_SVC_DB_LIST_TAG, uid);
	if (ret != MS_MEDIA_ERR_NONE) {
		media_svc_error("_media_svc_make_table_query fail.");
		goto ERROR;
	}

	/*create bookmark table*/
	ret = _media_svc_make_table_query(MEDIA_SVC_DB_TABLE_BOOKMARK, MEDIA_SVC_DB_LIST_BOOKMARK, uid);
	if (ret != MS_MEDIA_ERR_NONE) {
		media_svc_error("_media_svc_make_table_query fail.");
		goto ERROR;
	}

	/*create storage table from tizen 2.4 */
	ret = _media_svc_make_table_query(MEDIA_SVC_DB_TABLE_STORAGE, MEDIA_SVC_DB_LIST_STORAGE, uid);
	if (ret != MS_MEDIA_ERR_NONE) {
		media_svc_error("_media_svc_make_table_query fail.");
		goto ERROR;
	}

	/*create face table. from tizen 3.0*/
	ret = _media_svc_make_table_query(MEDIA_SVC_DB_TABLE_FACE_SCAN_LIST, MEDIA_SVC_DB_LIST_FACE_SCAN_LIST, uid);
	if (ret != MS_MEDIA_ERR_NONE) {
		media_svc_error("_media_svc_make_table_query fail.");
		goto ERROR;
	}

	ret = _media_svc_make_table_query(MEDIA_SVC_DB_TABLE_FACE, MEDIA_SVC_DB_LIST_FACE, uid);
	if (ret != MS_MEDIA_ERR_NONE) {
		media_svc_error("_media_svc_make_table_query fail.");
		goto ERROR;
	}

	sql = sqlite3_mprintf("pragma user_version = %d;", LATEST_VERSION_NUMBER);
	ret = _media_svc_sql_query(sql, uid);
	if (ret != MS_MEDIA_ERR_NONE) {
		media_svc_error("user_version update fail.");
		goto ERROR;
	}

	_media_svc_destroy_table_query();

	media_svc_debug_fleave();

	return MS_MEDIA_ERR_NONE;
ERROR:
	_media_svc_destroy_table_query();

	media_svc_debug_fleave();

	return ret;
}

int media_svc_check_item_exist_by_path(sqlite3 *handle, const char *storage_id, const char *path)
{
	int ret = MS_MEDIA_ERR_NONE;
	int count = -1;

	ret = _media_svc_count_record_with_path(handle, storage_id, path, &count);
	media_svc_retv_if(ret != MS_MEDIA_ERR_NONE, ret);

	if (count > 0) {
		media_svc_debug("item is exist in database");
		return MS_MEDIA_ERR_NONE;
	} else {
		media_svc_debug("item is not exist in database");
		return MS_MEDIA_ERR_DB_NO_RECORD;
	}

	return MS_MEDIA_ERR_NONE;
}

int media_svc_get_modified_time(sqlite3 *handle, const char *storage_id, const char *path, int *modified_time)
{
	return _media_svc_get_modified_time(handle, storage_id, path, modified_time);
}

int media_svc_insert_item_begin(int data_cnt, bool with_noti, int from_pid)
{
	media_svc_retvm_if(data_cnt < 1, MS_MEDIA_ERR_INVALID_PARAMETER, "data_cnt shuld be bigger than 1");

	g_media_svc_insert_item_data_cnt = data_cnt;
	g_media_svc_insert_item_cur_data_cnt = 0;

	/* Prepare for making noti item list */
	if (with_noti) {
		media_svc_debug("making noti list from pid[%d]", from_pid);
		if (_media_svc_create_noti_list(data_cnt) != MS_MEDIA_ERR_NONE)
			return MS_MEDIA_ERR_OUT_OF_MEMORY;

		_media_svc_set_noti_from_pid(from_pid);
		g_insert_with_noti = true;
	}

	return MS_MEDIA_ERR_NONE;
}

int media_svc_insert_item_end(uid_t uid)
{
	int ret = MS_MEDIA_ERR_NONE;

	media_svc_debug_fenter();

	if (g_media_svc_insert_item_cur_data_cnt > 0) {

		ret = _media_svc_list_query_do(MEDIA_SVC_QUERY_INSERT_ITEM, uid);
		if (g_insert_with_noti) {
			media_svc_debug("sending noti list");
			_media_svc_publish_noti_list(g_media_svc_insert_item_cur_data_cnt);
			_media_svc_destroy_noti_list(g_media_svc_insert_item_cur_data_cnt);
			g_insert_with_noti = false;
			_media_svc_set_noti_from_pid(-1);
		}
	}

	g_media_svc_insert_item_data_cnt = 1;
	g_media_svc_insert_item_cur_data_cnt = 0;

	return ret;
}

int media_svc_insert_item_bulk(sqlite3 *handle, const char *storage_id, ms_user_storage_type_e storage_type, const char *path, uid_t uid)
{
	int ret = MS_MEDIA_ERR_NONE;
	char folder_uuid[MEDIA_SVC_UUID_SIZE + 1] = {0, };

	media_svc_retvm_if(handle == NULL, MS_MEDIA_ERR_INVALID_PARAMETER, "Handle is NULL");
	media_svc_retvm_if(!STRING_VALID(storage_id), MS_MEDIA_ERR_INVALID_PARAMETER, "storage_id is NULL");
	media_svc_retvm_if(!STRING_VALID(path), MS_MEDIA_ERR_INVALID_PARAMETER, "path is NULL");
	media_svc_retvm_if(!_media_svc_is_valid_storage_type(storage_type), MS_MEDIA_ERR_INVALID_PARAMETER, "Invalid storage_type");

	media_svc_content_info_s content_info;
	memset(&content_info, 0, sizeof(media_svc_content_info_s));

	/*Set media info*/
	/* if drm_contentinfo is not NULL, the file is OMA DRM.*/
	ret = _media_svc_set_media_info(&content_info, storage_id, storage_type, path, false);
	if (ret != MS_MEDIA_ERR_NONE)
		return ret;

	switch (content_info.media_type) {
	case MEDIA_SVC_MEDIA_TYPE_IMAGE:
		ret = _media_svc_extract_image_metadata(&content_info);
		break;
	case MEDIA_SVC_MEDIA_TYPE_VIDEO:
	case MEDIA_SVC_MEDIA_TYPE_SOUND:
	case MEDIA_SVC_MEDIA_TYPE_MUSIC:
		ret = _media_svc_extract_media_metadata(handle, true, &content_info, uid);
		break;
	default:
		media_svc_debug("Do nothing[%d]", content_info.media_type);
		break;
	}

	media_svc_retv_del_if(ret != MS_MEDIA_ERR_NONE, ret, &content_info);

	/*Set or Get folder id*/
	ret = _media_svc_get_and_append_folder_id_by_path(handle, true, storage_id, path, storage_type, folder_uuid, uid);
	media_svc_retv_del_if(ret != MS_MEDIA_ERR_NONE, ret, &content_info);

	content_info.folder_uuid = g_strdup(folder_uuid);
	media_svc_retv_del_if(content_info.folder_uuid == NULL, MS_MEDIA_ERR_INTERNAL, &content_info);

	if (g_media_svc_insert_item_data_cnt == 1) {

		ret = _media_svc_insert_item_with_data(true, storage_id, &content_info, false, uid);
		media_svc_retv_del_if(ret != MS_MEDIA_ERR_NONE, ret, &content_info);

		if (g_insert_with_noti)
			_media_svc_insert_item_to_noti_list(&content_info, g_media_svc_insert_item_cur_data_cnt++);

	} else if (g_media_svc_insert_item_cur_data_cnt < (g_media_svc_insert_item_data_cnt - 1)) {

		ret = _media_svc_insert_item_with_data(true, storage_id, &content_info, true, uid);
		media_svc_retv_del_if(ret != MS_MEDIA_ERR_NONE, ret, &content_info);

		if (g_insert_with_noti)
			_media_svc_insert_item_to_noti_list(&content_info, g_media_svc_insert_item_cur_data_cnt);

		g_media_svc_insert_item_cur_data_cnt++;

	} else if (g_media_svc_insert_item_cur_data_cnt == (g_media_svc_insert_item_data_cnt - 1)) {

		ret = _media_svc_insert_item_with_data(true, storage_id, &content_info, true, uid);
		media_svc_retv_del_if(ret != MS_MEDIA_ERR_NONE, ret, &content_info);

		if (g_insert_with_noti)
			_media_svc_insert_item_to_noti_list(&content_info, g_media_svc_insert_item_cur_data_cnt);

		ret = _media_svc_list_query_do(MEDIA_SVC_QUERY_INSERT_ITEM, uid);
		media_svc_retv_del_if(ret != MS_MEDIA_ERR_NONE, ret, &content_info);

		if (g_insert_with_noti) {
			_media_svc_publish_noti_list(g_media_svc_insert_item_cur_data_cnt + 1);
			_media_svc_destroy_noti_list(g_media_svc_insert_item_cur_data_cnt + 1);

			/* Recreate noti list */
			ret = _media_svc_create_noti_list(g_media_svc_insert_item_data_cnt);
			media_svc_retv_del_if(ret != MS_MEDIA_ERR_NONE, ret, &content_info);
		}

		g_media_svc_insert_item_cur_data_cnt = 0;

	} else {
		media_svc_error("Error in media_svc_insert_item_bulk");
		_media_svc_destroy_content_info(&content_info);
		return MS_MEDIA_ERR_INTERNAL;
	}

	_media_svc_destroy_content_info(&content_info);

	return MS_MEDIA_ERR_NONE;
}

int media_svc_insert_item_immediately(sqlite3 *handle, const char *storage_id, ms_user_storage_type_e storage_type, const char *path, uid_t uid)
{
	int ret = MS_MEDIA_ERR_NONE;
	char folder_uuid[MEDIA_SVC_UUID_SIZE + 1] = {0, };

	media_svc_retvm_if(handle == NULL, MS_MEDIA_ERR_INVALID_PARAMETER, "Handle is NULL");
	media_svc_retvm_if(!STRING_VALID(storage_id), MS_MEDIA_ERR_INVALID_PARAMETER, "storage_id is NULL");
	media_svc_retvm_if(!STRING_VALID(path), MS_MEDIA_ERR_INVALID_PARAMETER, "path is NULL");
	media_svc_retvm_if(!_media_svc_is_valid_storage_type(storage_type), MS_MEDIA_ERR_INVALID_PARAMETER, "Invalid storage_type");

	media_svc_content_info_s content_info;
	memset(&content_info, 0, sizeof(media_svc_content_info_s));

	/*Set media info*/
	ret = _media_svc_set_media_info(&content_info, storage_id, storage_type, path, false);
	if (ret != MS_MEDIA_ERR_NONE)
		return ret;

	switch (content_info.media_type) {
	case MEDIA_SVC_MEDIA_TYPE_IMAGE:
		ret = _media_svc_extract_image_metadata(&content_info);
		break;
	case MEDIA_SVC_MEDIA_TYPE_VIDEO:
	case MEDIA_SVC_MEDIA_TYPE_SOUND:
	case MEDIA_SVC_MEDIA_TYPE_MUSIC:
		ret = _media_svc_extract_media_metadata(handle, false, &content_info, uid);
		break;
	default:
		media_svc_debug("Do nothing[%d]", content_info.media_type);
		break;
	}

	media_svc_retv_del_if(ret != MS_MEDIA_ERR_NONE, ret, &content_info);

	/*Set or Get folder id*/
	ret = _media_svc_get_and_append_folder_id_by_path(handle, false, storage_id, path, storage_type, folder_uuid, uid);
	media_svc_retv_del_if(ret != MS_MEDIA_ERR_NONE, ret, &content_info);

	content_info.folder_uuid = g_strdup(folder_uuid);
	media_svc_retv_del_if(content_info.folder_uuid == NULL, MS_MEDIA_ERR_INTERNAL, &content_info);

	/* Extracting thumbnail */
	if (content_info.thumbnail_path == NULL) {
		if (content_info.media_type == MEDIA_SVC_MEDIA_TYPE_IMAGE || content_info.media_type == MEDIA_SVC_MEDIA_TYPE_VIDEO) {
			char thumb_path[MEDIA_SVC_PATHNAME_SIZE + 1] = {0, };

			ret = _media_svc_create_thumbnail(content_info.path, thumb_path, content_info.media_type, uid);
			if (ret == MS_MEDIA_ERR_NONE)
				content_info.thumbnail_path = g_strdup(thumb_path);
		}
	}

	ret = _media_svc_insert_item_with_data(false, storage_id, &content_info, false, uid);

	if (ret == MS_MEDIA_ERR_NONE) {
		media_svc_debug("Insertion is successful. Sending noti for this");
		_media_svc_publish_noti(MS_MEDIA_ITEM_INSERT, content_info.path, content_info.media_type, content_info.media_uuid, content_info.mime_type);
	} else if (ret == MS_MEDIA_ERR_DB_CONSTRAINT_FAIL) {
		media_svc_error("This item is already inserted. This may be normal operation because other process already did this");
	}

	_media_svc_destroy_content_info(&content_info);
	return ret;
}

int media_svc_move_item(sqlite3 *handle,
								const char *src_path,
								const char *dest_path,
								const char *media_id,
								int media_type,
								const char *mime_type,
								uid_t uid)
{
	int ret = MS_MEDIA_ERR_NONE;
	char *file_name = NULL;
	char *folder_path = NULL;
	int modified_time = 0;
	char folder_uuid[MEDIA_SVC_UUID_SIZE + 1] = {0, };
	char old_thumb_path[MEDIA_SVC_PATHNAME_SIZE] = {0, };
	char org_stg_id[MEDIA_SVC_UUID_SIZE + 1] = {0, };
	char dst_stg_id[MEDIA_SVC_UUID_SIZE + 1] = {0, };
	ms_user_storage_type_e org_stg_type = MS_USER_STORAGE_INTERNAL;
	ms_user_storage_type_e dst_stg_type = MS_USER_STORAGE_INTERNAL;

	media_svc_debug_fenter();

	media_svc_retvm_if(handle == NULL, MS_MEDIA_ERR_INVALID_PARAMETER, "Handle is NULL");
	media_svc_retvm_if(!STRING_VALID(src_path), MS_MEDIA_ERR_INVALID_PARAMETER, "src_path is NULL");
	media_svc_retvm_if(!STRING_VALID(dest_path), MS_MEDIA_ERR_INVALID_PARAMETER, "dest_path is NULL");
	media_svc_retvm_if(!STRING_VALID(media_id), MS_MEDIA_ERR_INVALID_PARAMETER, "media_id is NULL");
	media_svc_retvm_if(!STRING_VALID(mime_type), MS_MEDIA_ERR_INVALID_PARAMETER, "mime_type is NULL");

	/* Get storage_id */
	ret = _media_svc_get_storage_uuid(handle, src_path, org_stg_id, uid);
	media_svc_retv_if(ret != MS_MEDIA_ERR_NONE, ret);
	ret = _media_svc_get_storage_uuid(handle, dest_path, dst_stg_id, uid);
	media_svc_retv_if(ret != MS_MEDIA_ERR_NONE, ret);
	/* Get storage_type */
	ret = ms_user_get_storage_type(uid, src_path, &org_stg_type);
	media_svc_retv_if(ret != MS_MEDIA_ERR_NONE, ret);
	ret = ms_user_get_storage_type(uid, dest_path, &dst_stg_type);
	media_svc_retv_if(ret != MS_MEDIA_ERR_NONE, ret);

	/*check and update folder*/
	ret = _media_svc_get_and_append_folder_id_by_path(handle, false, dst_stg_id, dest_path, dst_stg_type, folder_uuid, uid);
	media_svc_retv_if(ret != MS_MEDIA_ERR_NONE, ret);

	/*get filename*/
	file_name = g_path_get_basename(dest_path);

	/*get modified_time*/
	modified_time = _media_svc_get_file_time(dest_path);

	/*get old thumbnail_path and remove thumbnail */
	ret = _media_svc_get_thumbnail_path_by_path(handle, src_path, old_thumb_path);
	if ((ret != MS_MEDIA_ERR_NONE) && (ret != MS_MEDIA_ERR_DB_NO_RECORD)) {
		media_svc_error("_media_svc_get_thumbnail_path_by_path failed");
		SAFE_FREE(file_name);
		return ret;
	}

	if (STRING_VALID(old_thumb_path)) {
		ret = _media_svc_remove_file(old_thumb_path);
		if (ret != MS_MEDIA_ERR_NONE)
			media_svc_error("_media_svc_remove_file failed : %d", ret);
	}

	/*move item*/
	ret = _media_svc_update_item_by_path(org_stg_id, src_path, dst_stg_id, dst_stg_type, dest_path, file_name, modified_time, folder_uuid, uid);
	SAFE_FREE(file_name);
	media_svc_retv_if(ret != MS_MEDIA_ERR_NONE, ret);

	media_svc_debug("Move is successful. Sending noti for this");
	_media_svc_publish_noti(MS_MEDIA_ITEM_UPDATE, src_path, media_type, media_id, mime_type);

	/*update folder modified_time*/
	folder_path = g_path_get_dirname(dest_path);
	ret = _media_svc_update_folder_modified_time_by_folder_uuid(folder_uuid, folder_path, uid);
	SAFE_FREE(folder_path);
	media_svc_retv_if(ret != MS_MEDIA_ERR_NONE, ret);

	return MS_MEDIA_ERR_NONE;
}

int media_svc_set_item_validity_begin(int data_cnt)
{
	media_svc_debug("Transaction data count : [%d]", data_cnt);

	media_svc_retvm_if(data_cnt < 1, MS_MEDIA_ERR_INVALID_PARAMETER, "data_cnt shuld be bigger than 1");

	g_media_svc_item_validity_data_cnt = data_cnt;
	g_media_svc_item_validity_cur_data_cnt = 0;

	return MS_MEDIA_ERR_NONE;
}

int media_svc_set_item_validity_end(uid_t uid)
{
	int ret = MS_MEDIA_ERR_NONE;

	media_svc_debug_fenter();

	if (g_media_svc_item_validity_cur_data_cnt > 0)
		ret = _media_svc_list_query_do(MEDIA_SVC_QUERY_SET_ITEM_VALIDITY, uid);

	g_media_svc_item_validity_data_cnt = 1;
	g_media_svc_item_validity_cur_data_cnt = 0;

	return ret;
}

int media_svc_set_item_validity(const char *storage_id, const char *path, int validity, uid_t uid)
{
	int ret = MS_MEDIA_ERR_NONE;

	if (g_media_svc_item_validity_data_cnt == 1) {

		return _media_svc_update_item_validity(storage_id, path, validity, false, uid);

	} else if (g_media_svc_item_validity_cur_data_cnt < (g_media_svc_item_validity_data_cnt - 1)) {

		ret = _media_svc_update_item_validity(storage_id, path, validity, true, uid);
		media_svc_retv_if(ret != MS_MEDIA_ERR_NONE, ret);

		g_media_svc_item_validity_cur_data_cnt++;

	} else if (g_media_svc_item_validity_cur_data_cnt == (g_media_svc_item_validity_data_cnt - 1)) {

		ret = _media_svc_update_item_validity(storage_id, path, validity, true, uid);
		media_svc_retv_if(ret != MS_MEDIA_ERR_NONE, ret);

		ret = _media_svc_list_query_do(MEDIA_SVC_QUERY_SET_ITEM_VALIDITY, uid);
		media_svc_retv_if(ret != MS_MEDIA_ERR_NONE, ret);

		g_media_svc_item_validity_cur_data_cnt = 0;

	} else {

		media_svc_error("Error in media_svc_set_item_validity");
		return MS_MEDIA_ERR_INTERNAL;
	}

	return MS_MEDIA_ERR_NONE;
}

int media_svc_delete_item_by_path(sqlite3 *handle, const char *storage_id, const char *path, uid_t uid)
{
	int ret = MS_MEDIA_ERR_NONE;
	char thumb_path[MEDIA_SVC_PATHNAME_SIZE] = {0, };
	media_svc_noti_item *noti_item = NULL;

	media_svc_debug_fenter();

	/*Get thumbnail path to delete*/
	ret = _media_svc_get_thumbnail_path_by_path(handle, path, thumb_path);
	media_svc_retv_if((ret != MS_MEDIA_ERR_NONE) && (ret != MS_MEDIA_ERR_DB_NO_RECORD), ret);

	/* Get notification info */
	ret = _media_svc_get_noti_info(handle, storage_id, path, &noti_item);
	media_svc_retv_if(ret != MS_MEDIA_ERR_NONE, ret);

	/*Delete item*/
	ret = _media_svc_delete_item_by_path(storage_id, path, uid);
	if (ret != MS_MEDIA_ERR_NONE) {
		media_svc_error("_media_svc_delete_item_by_path failed : %d", ret);
		_media_svc_destroy_noti_item(noti_item);

		return ret;
	}

	/* Send notification */
	media_svc_debug("Deletion is successful. Sending noti for this");
	_media_svc_publish_noti(MS_MEDIA_ITEM_DELETE, path, noti_item->media_type, noti_item->media_uuid, noti_item->mime_type);
	_media_svc_destroy_noti_item(noti_item);

	/*Delete thumbnail*/
	if (STRING_VALID(thumb_path)) {
		ret = _media_svc_remove_file(thumb_path);
		if (ret != MS_MEDIA_ERR_NONE)
			media_svc_error("fail to remove thumbnail file.");
	}

	return MS_MEDIA_ERR_NONE;
}

int media_svc_refresh_item(sqlite3 *handle, bool is_direct, const char *storage_id, ms_user_storage_type_e storage_type, const char *path, uid_t uid)
{
	int ret = MS_MEDIA_ERR_NONE;
	char thumb_path[MEDIA_SVC_PATHNAME_SIZE + 1] = {0, };

	media_svc_retvm_if(handle == NULL, MS_MEDIA_ERR_INVALID_PARAMETER, "Handle is NULL");
	media_svc_retvm_if(!STRING_VALID(storage_id), MS_MEDIA_ERR_INVALID_PARAMETER, "storage_id is NULL");
	media_svc_retvm_if(!STRING_VALID(path), MS_MEDIA_ERR_INVALID_PARAMETER, "path is NULL");
	media_svc_retvm_if(!_media_svc_is_valid_storage_type(storage_type), MS_MEDIA_ERR_INVALID_PARAMETER, "Invalid storage_type");

	media_svc_content_info_s content_info;
	memset(&content_info, 0, sizeof(media_svc_content_info_s));

	/*Set media info*/
	ret = _media_svc_set_media_info(&content_info, storage_id, storage_type, path, true);
	if (ret != MS_MEDIA_ERR_NONE)
		return ret;

	/* Initialize thumbnail information to remake thumbnail. */
	ret = _media_svc_get_thumbnail_path_by_path(handle, path, thumb_path);
	if (ret != MS_MEDIA_ERR_NONE && ret != MS_MEDIA_ERR_DB_NO_RECORD) {
		_media_svc_destroy_content_info(&content_info);
		return ret;
	}

	if (STRING_VALID(thumb_path)) {
		if (g_file_test(thumb_path, G_FILE_TEST_EXISTS)) {
			ret = _media_svc_remove_file(thumb_path);
			if (ret != MS_MEDIA_ERR_NONE)
				media_svc_error("_media_svc_remove_file failed : %s", thumb_path);
		}

		ret = _media_svc_update_thumbnail_path(storage_id, path, NULL, uid);
		if (ret != MS_MEDIA_ERR_NONE) {
			_media_svc_destroy_content_info(&content_info);
			return ret;
		}
	}

	/* Get notification info */
	media_svc_noti_item *noti_item = NULL;
	ret = _media_svc_get_noti_info(handle, storage_id, path, &noti_item);
	if (ret != MS_MEDIA_ERR_NONE) {
		_media_svc_destroy_content_info(&content_info);
		return ret;
	}

	content_info.media_type = noti_item->media_type;

	if (content_info.media_type == MEDIA_SVC_MEDIA_TYPE_OTHER
	|| (content_info.media_type == MEDIA_SVC_MEDIA_TYPE_PVR)
	|| (content_info.media_type == MEDIA_SVC_MEDIA_TYPE_UHD)
	|| (content_info.media_type == MEDIA_SVC_MEDIA_TYPE_SCSA))
		media_svc_debug("Do nothing [%d]", content_info.media_type);
	else if (content_info.media_type == MEDIA_SVC_MEDIA_TYPE_IMAGE)
		ret = _media_svc_extract_image_metadata(&content_info);
	else
		ret = _media_svc_extract_media_metadata(handle, is_direct, &content_info, uid);

	if (ret != MS_MEDIA_ERR_NONE) {
		_media_svc_destroy_noti_item(noti_item);
		_media_svc_destroy_content_info(&content_info);
		return ret;
	}

	/* Extracting thumbnail */
	if (content_info.thumbnail_path == NULL) {
		if (content_info.media_type == MEDIA_SVC_MEDIA_TYPE_IMAGE || content_info.media_type == MEDIA_SVC_MEDIA_TYPE_VIDEO) {
			memset(thumb_path, 0, sizeof(thumb_path));

			ret = _media_svc_create_thumbnail(content_info.path, thumb_path, content_info.media_type, uid);
			if (ret == MS_MEDIA_ERR_NONE)
				content_info.thumbnail_path = g_strdup(thumb_path);
		}
	}

	ret = _media_svc_update_item_with_data(is_direct, storage_id, &content_info, uid);

	if (ret == MS_MEDIA_ERR_NONE) {
		media_svc_debug("Update is successful. Sending noti for this");
		_media_svc_publish_noti(MS_MEDIA_ITEM_UPDATE, content_info.path, noti_item->media_type, noti_item->media_uuid, noti_item->mime_type);
	} else {
		media_svc_error("_media_svc_update_item_with_data failed : %d", ret);
	}

	_media_svc_destroy_content_info(&content_info);
	_media_svc_destroy_noti_item(noti_item);

	return ret;
}

int media_svc_send_dir_update_noti(const char *dir_path, const char *folder_id, media_item_update_type_e update_type, int pid)
{
	media_svc_retvm_if(!STRING_VALID(dir_path), MS_MEDIA_ERR_INVALID_PARAMETER, "dir_path is NULL");

	return _media_svc_publish_dir_noti(update_type, dir_path, folder_id, pid);
}

int media_svc_check_db_upgrade(sqlite3 *handle, int user_version, uid_t uid)
{
	media_svc_debug_fenter();

	return _media_svc_check_db_upgrade(handle, user_version, uid);
}

static void __media_svc_noti_all_storage(sqlite3 *handle, uid_t uid)
{
	int ret = MS_MEDIA_ERR_NONE;
	char *root_path = NULL;
	GPtrArray *path_list = NULL;
	int i = 0;

	ret = ms_user_get_internal_root_path(uid, &root_path);
	media_svc_retm_if(ret != MS_MEDIA_ERR_NONE, "Fail to get root path");

	ret = _media_svc_publish_dir_noti(MS_MEDIA_ITEM_UPDATE, root_path, NULL, 0);
	if (ret != MS_MEDIA_ERR_NONE)
		media_svc_error("Fail to send noti");

	SAFE_FREE(root_path);

	path_list = g_ptr_array_new_with_free_func(g_free);
	_media_svc_get_storage_path(handle, &path_list);

	for (i = 0; i < path_list->len; i++) {
		root_path = g_ptr_array_index(path_list, i);

		ret = _media_svc_publish_dir_noti(MS_MEDIA_ITEM_UPDATE, root_path, NULL, 0);
		if (ret != MS_MEDIA_ERR_NONE)
			media_svc_error("Fail to send noti");
	}

	g_ptr_array_free(path_list, TRUE);
}

static void __file_info_free(gpointer data)
{
	SAFE_FREE(*(media_svc_file_s **)data);
}

int media_svc_update_item_meta(sqlite3 *handle, uid_t uid)
{
	int ret = MS_MEDIA_ERR_NONE;
	int i = 0;
	/* NOTICE : After 6.0, change 'media_view' to 'media', add 'AND validity=1', and no need storage id */
	char *sql = NULL;
	media_svc_content_info_s content_info;
	media_svc_file_s *file_info = NULL;
	GArray *path_list = NULL;

	path_list = g_array_new(FALSE, FALSE, sizeof(media_svc_file_s *));
	media_svc_retvm_if(!path_list, MS_MEDIA_ERR_OUT_OF_MEMORY, "Allocation failed");
	g_array_set_clear_func(path_list, __file_info_free);

	sql = sqlite3_mprintf("SELECT media_path, storage_uuid FROM %q WHERE media_type=3", MEDIA_SVC_DB_VIEW_MEDIA);
	ret = _media_svc_get_media(handle, sql, &path_list);
	if (ret != MS_MEDIA_ERR_NONE) {
		media_svc_error("Fail to get media list");
		g_array_free(path_list, TRUE);
		return ret;
	}

	for (i = 0; i < path_list->len; i++) {
		file_info = g_array_index(path_list, media_svc_file_s *, i);

		memset(&content_info, 0, sizeof(media_svc_content_info_s));
		ret = _media_svc_extract_music_metadata_for_update(&content_info, file_info->storage_id, file_info->path);
		if (ret != MS_MEDIA_ERR_NONE) {
			media_svc_error("Fail to extract metadata");
			_media_svc_destroy_content_info(&content_info);
			continue;
		}

		ret = _media_svc_update_meta_with_data(&content_info);
		if (ret != MS_MEDIA_ERR_NONE)
			media_svc_error("Fail to append item[%s]", content_info.path);

		_media_svc_destroy_content_info(&content_info);
	}

	g_array_free(path_list, TRUE);

	ret = _media_svc_list_query_do(MEDIA_SVC_QUERY_UPDATE_ITEM, uid);
	media_svc_retvm_if(ret != MS_MEDIA_ERR_NONE, ret, "_media_svc_list_query_do failed");

	/* Noti for this */
	__media_svc_noti_all_storage(handle, uid);

	return ret;
}

int media_svc_publish_noti(media_item_update_type_e update_type, const char *path, media_type_e media_type, const char *uuid, const char *mime_type)
{
	return _media_svc_publish_noti(update_type, path, media_type, uuid, mime_type);
}

int media_svc_get_pinyin(const char *src_str, char **pinyin_str)
{
	return _media_svc_get_pinyin_str(src_str, pinyin_str);
}

int media_svc_check_pinyin_support(bool *support)
{
	*support = _media_svc_check_pinyin_support();

	return MS_MEDIA_ERR_NONE;
}

int media_svc_set_storage_validity(sqlite3 *handle, const char *storage_id, int validity, uid_t uid)
{
	int ret = MS_MEDIA_ERR_NONE;

	media_svc_retvm_if(handle == NULL, MS_MEDIA_ERR_INVALID_PARAMETER, "Handle is NULL");

	ret = _media_svc_update_storage_validity(storage_id, validity, uid);
	media_svc_retvm_if(ret != MS_MEDIA_ERR_NONE, ret, "update storage validity failed: %d", ret);

	ret = _media_svc_update_media_view(handle, uid);
	media_svc_retvm_if(ret != MS_MEDIA_ERR_NONE, ret, "update media view failed : %d", ret);

	return ret;
}

int media_svc_get_storage_id(sqlite3 *handle, const char *path, char *storage_id, uid_t uid)
{
	return _media_svc_get_storage_uuid(handle, path, storage_id, uid);
}

int media_svc_generate_uuid(char **uuid)
{
	char *gen_uuid = NULL;
	gen_uuid = _media_info_generate_uuid();
	media_svc_retvm_if(gen_uuid == NULL, MS_MEDIA_ERR_INTERNAL, "Fail to generate uuid");

	*uuid = strdup(gen_uuid);

	return MS_MEDIA_ERR_NONE;
}

int media_svc_check_storage(sqlite3 *handle, const char *storage_id, char **storage_path, int *validity, uid_t uid)
{
	return _media_svc_check_storage(handle, storage_id, storage_path, validity, uid);
}

int media_svc_update_storage(sqlite3 *handle, const char *storage_id, const char *storage_path, uid_t uid)
{
	return _media_svc_update_storage_path(handle, storage_id, storage_path, uid);
}

int media_svc_insert_storage(sqlite3 *handle, const char *storage_id, const char *storage_path, ms_user_storage_type_e storage_type, uid_t uid)
{
	int ret = MS_MEDIA_ERR_NONE;

	media_svc_retvm_if(handle == NULL, MS_MEDIA_ERR_INVALID_PARAMETER, "Handle is NULL");
	media_svc_retvm_if(storage_id == NULL, MS_MEDIA_ERR_INVALID_PARAMETER, "storage_id is NULL");
	media_svc_retvm_if(storage_path == NULL, MS_MEDIA_ERR_INVALID_PARAMETER, "storage_path is NULL");
	media_svc_retvm_if(!_media_svc_is_valid_storage_type(storage_type), MS_MEDIA_ERR_INVALID_PARAMETER, "Invalid storage_type");

	ret = _media_svc_append_storage(storage_id, storage_path, storage_type, uid);
	media_svc_retvm_if(ret != MS_MEDIA_ERR_NONE, ret, "append storage failed : %d", ret);

	ret = _media_svc_create_media_table_with_id(storage_id, uid);
	media_svc_retvm_if(ret != MS_MEDIA_ERR_NONE, ret, "create media table failed : %d", ret);

	ret = _media_svc_update_media_view(handle, uid);
	media_svc_retvm_if(ret != MS_MEDIA_ERR_NONE, ret, "update media view failed : %d", ret);

	/* Remove external storage that validity is 0 */
	ret = _media_svc_delete_invalid_storage(handle, uid);
	media_svc_retvm_if(ret != MS_MEDIA_ERR_NONE, ret, "Delete invalid storage failed : %d", ret);

	return ret;
}

int media_svc_insert_folder(sqlite3 *handle, const char *storage_id, ms_user_storage_type_e storage_type, const char *path, uid_t uid)
{
	media_svc_retvm_if(handle == NULL, MS_MEDIA_ERR_INVALID_PARAMETER, "Handle is NULL");
	media_svc_retvm_if(!STRING_VALID(storage_id), MS_MEDIA_ERR_INVALID_PARAMETER, "storage_id is NULL");
	media_svc_retvm_if(!STRING_VALID(path), MS_MEDIA_ERR_INVALID_PARAMETER, "path is NULL");
	media_svc_retvm_if(!_media_svc_is_valid_storage_type(storage_type), MS_MEDIA_ERR_INVALID_PARAMETER, "Invalid storage_type");

	return _media_svc_get_and_append_folder_id_by_folder_path(handle, storage_id, path, storage_type, uid);
}

int media_svc_set_folder_validity(sqlite3 *handle, const char *storage_id, const char *start_path, int validity, bool is_recursive, uid_t uid)
{
	return _media_svc_set_folder_validity(handle, true, storage_id, start_path, validity, is_recursive, uid);
}

int media_svc_check_folder_exist_by_path(sqlite3 *handle, const char *storage_id, const char *folder_path)
{
	int ret = MS_MEDIA_ERR_NONE;
	int count = -1;

	ret = _media_svc_count_folder_with_path(handle, storage_id, folder_path, &count);
	media_svc_retv_if(ret != MS_MEDIA_ERR_NONE, ret);

	if (count > 0) {
		media_svc_debug("item is exist in database");
		return MS_MEDIA_ERR_NONE;
	} else {
		media_svc_debug("item is not exist in database");
		return MS_MEDIA_ERR_DB_NO_RECORD;
	}

	return MS_MEDIA_ERR_NONE;
}

int media_svc_append_query(const char *query, uid_t uid)
{
	return _media_svc_append_query_list(query, uid);
}

int media_svc_send_query(uid_t uid)
{
	return _media_svc_list_query_do(MEDIA_SVC_QUERY_UPDATE_COMMON, uid);
}

int media_svc_get_media_type(const char *path, int *mediatype)
{
	return _media_svc_get_media_type(path, mediatype);
}

int media_svc_create_thumbnail(const char *storage_id, const char *file_path, int media_type, uid_t uid, char **thumbnail_path)
{
	int ret = MS_MEDIA_ERR_NONE;
	char thumb_path[MEDIA_SVC_PATHNAME_SIZE + 1] = { 0, };
	char *sql = NULL;

	// 1. Check media type
	if (media_type != MS_MEDIA_IMAGE && media_type != MS_MEDIA_VIDEO)
		return MS_MEDIA_ERR_UNSUPPORTED_CONTENT;

	// 2. try to create thumbnail
	ret = _media_svc_create_thumbnail(file_path, thumb_path, media_type, uid);
	if (ret != MS_MEDIA_ERR_NONE) {
		media_svc_error("Failed to create thumbnail [%d]", ret);
		if (ret == MS_MEDIA_ERR_UNSUPPORTED_CONTENT)
			return ret;
	}

	// 3. Update creation result to media db
	sql = sqlite3_mprintf("UPDATE '%q' SET media_thumbnail_path='%q' WHERE media_path='%q';", storage_id, thumb_path, file_path);

	ret = _media_svc_sql_query(sql, uid);
	SQLITE3_SAFE_FREE(sql);
	if (ret != MS_MEDIA_ERR_NONE) {
		media_svc_error("Failed to update media db [%d]", ret);
		*thumbnail_path = g_strdup("");
	} else {
		*thumbnail_path = g_strdup(thumb_path);
	}

	return ret;
}
